/*
 * Bootloader version of the i2c driver for AIDA FEE64 board
 * Reads Microchip 24AA02E48 MAC; extracts MAC address and sets property 
      local-mac-address in the device tree
 *
*/

#include <stdarg.h>
#include <stddef.h>
#include "types.h"
#include "elf.h"
#include "page.h"
#include "string.h"
#include "stdio.h"
#include "io.h"
#include "ops.h"

int xaidaeeprom_i2c_open(void);
void xaidaeeprom_i2c_close(void);
int xaidaeeprom_i2c_read(void);

#define XIIC_REG_OFFSET  0x100
#define XIIC_CR_REG_OFFSET 0x00+XIIC_REG_OFFSET   /* Control register */
#define XIIC_SR_REG_OFFSET 0x04+XIIC_REG_OFFSET   /* Status register */
#define XIIC_DTR_REG_OFFSET 0x08+XIIC_REG_OFFSET  /* Data TX register */
#define XIIC_DRR_REG_OFFSET 0x0c+XIIC_REG_OFFSET  /* Data RX register */
#define XIIC_RFD_REG_OFFSET 0x20+XIIC_REG_OFFSET

#define XIIC_SR_BUS_BUSY_MASK 0x04
#define XIIC_SR_RX_FIFO_EMPTY_MASK 0x40

#define IIC_ADDR 0x50
#define MAC_ADDR 0xfa

#define MAXTRIES 1000

static u8 *iic_base = NULL;
static u32 iic_offset = IIC_ADDR;
static u32 iic_mac_offset = MAC_ADDR;

static unsigned char iic_macaddr[6] = {255,255,255,255,255,255};

int xaidaeeprom_i2c_read(void)
{
	int i;
	int data;
	int status;
        int tries;

    data = 15;
    out_be32((u32 *)(iic_base + XIIC_RFD_REG_OFFSET), data);  //   set Rx_FIFO depth

    data = 2;       //  reset Tx_FIFO
    out_be32((u32 *)(iic_base + XIIC_CR_REG_OFFSET), data);   //  control register
 
    data = 1;       //  enable; remove Tx_FIFO reset
    out_be32((u32 *)(iic_base + XIIC_CR_REG_OFFSET), data);  //  control register

    status = in_be32((u32 *)(iic_base + XIIC_SR_REG_OFFSET));      //   status register 
    if ((status & XIIC_SR_BUS_BUSY_MASK) != 0) {
        printf("iic bus is busy: status= 0x%x\r\n", status);
        return 1;
    }

    data = 9;       //  enable; transmit
    out_be32((u32 *)(iic_base + XIIC_CR_REG_OFFSET), data);   //  control register

    data = (iic_offset << 1) + 0 + 0x100;      /*  set start bit; device address; write access */
    out_be32((u32 *)(iic_base + XIIC_DTR_REG_OFFSET), data);   //  data TX register

    data = iic_mac_offset;      /*   MAC address */
    out_be32((u32 *)(iic_base + XIIC_DTR_REG_OFFSET), data);   //  data TX register

    data = 5;       //  receive + MSMS=1 + enable
    out_be32((u32 *)(iic_base + XIIC_CR_REG_OFFSET), data);  //  control register

    data = (iic_offset << 1) + 1 + 0x100;      //  set start bit for repeated start; device address; read access
    out_be32((u32 *)(iic_base + XIIC_DTR_REG_OFFSET), data);   //  data TX register

    data = 0x200 + 6;      //  set stop bit; 6 bytes to be received
    out_be32((u32 *)(iic_base + XIIC_DTR_REG_OFFSET), data);   //  data TX register

    i = 0;
    for (tries=MAXTRIES; tries>0; tries--) {

        status = in_be32((u32 *)(iic_base + XIIC_SR_REG_OFFSET));      //   status register 
        if (!(status & XIIC_SR_RX_FIFO_EMPTY_MASK)) {

            data = in_be32((u32 *)(iic_base + XIIC_DRR_REG_OFFSET));      //   data RX register 
            iic_macaddr[i] = (unsigned char) (data & 255);
            i++;
        } else {
            udelay(10);
        }
        if (i == 6) break;
    }
    if (tries == 0) {
      printf("Failed to obtain MAC address: ");
      printf("read status register: status= 0x%x\r\n", status);
      return 1;
    }

    return 0;
}

int xaidaeeprom_i2c_open(void)
{
	u32 v;
	void *devp;
        int rc=-1;
        int i;

	devp = find_node_by_alias("xaidaeeprom");
	if (devp != NULL) {
	   if (getprop(devp, "iic", &v, sizeof(v)) == sizeof(v)) iic_base = (u8 *)v;
	   if (getprop(devp, "reg", &v, sizeof(v)) == sizeof(v)) iic_offset = v;
	   if (getprop(devp, "mac", &v, sizeof(v)) == sizeof(v)) iic_mac_offset = v;
        }

       if (iic_base == NULL) return rc;

      printf("Probing IIC bus for MAC... ");

        rc = xaidaeeprom_i2c_read();
       if (rc != 0) return rc;

      printf("MAC address = ");
       for (i=0; i<6; i++) {printf("0x%x ",iic_macaddr[i]);}
       printf("\r\n");

        devp = find_node_by_alias("ethernet0");
        if (devp != NULL) {
           setprop(devp, "local-mac-address", iic_macaddr, 6);
        }

	return rc;
}

void xaidaeeprom_i2c_close(void)
{
     iic_base = NULL;
}


